#include "stdafx.h"
#include "TaskAmpHistogram.h"

BOOL CALLBACK VwAmpHistogramDlgProc( HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam) ;

//-- Global data 
char * g_szTitleAmpHist = "Amplitude Histogram";

//-----------------------------------------------------------------------------
// dialog for options edit 
class CVwAmpHistogram : public CDialog { 
	private : 
		//-- Controls 
		CDlgCombo m_cmbChannel;
				
		CDlgCheck m_selDataFile;
		CDlgCheck m_selDataList;
		CDlgCheck m_selDataCurrent;
		
		CDlgEdit m_edtBinCount;		
		bool m_bDataCurrent;
		bool m_bDataList;
		
	public : 		
		COptionsAmpHistogram * m_pConfig;		// ptr to structure to store control values when done

		CVwAmpHistogram( ) {
			m_iID = IDD_TASK_AMPHISTOGRAM;
			m_pConfig = NULL;
			m_bDataCurrent = true;
			m_bDataList = true;
			}
	
		~CVwAmpHistogram() { 
			}

		//-- Initialize controls 
		void InitializeControls( ) { 
			m_cmbChannel.SetHandle( GetDlgItem(m_hDlg, IDC_SELCHANNEL), "Select the data channel to analyze");

			m_selDataFile.SetHandle( GetDlgItem(m_hDlg, IDC_DATASEL1), "Histogram for entire file");
			m_selDataList.SetHandle( GetDlgItem(m_hDlg, IDC_DATASEL2), "Histogram for the current selection list only");
			m_selDataCurrent.SetHandle( GetDlgItem(m_hDlg, IDC_DATASEL3), "Histogram for the current selection only");
			
			m_edtBinCount.SetHandle( GetDlgItem(m_hDlg, IDC_EDTBINCOUNT), "Number of groups to divide data into,\nwhich determines the number of bars displayed in the graph.");

			if( ! m_bDataCurrent  )
				EnableWindow( m_selDataCurrent.GetHandle(), false );
			if( ! m_bDataList ) 
				EnableWindow( m_selDataList.GetHandle(), false );
			}

		//-- Set all control contents and disable unused rows
		void SetControlValues( ) { 
			//-- Channel numbers
			char buf[64];
			m_cmbChannel.ClearContents();
			for( int iCh=0; iCh<m_pConfig->m_iChannels; ++iCh ) {
				sprintf( buf, "%d", iCh );
				m_cmbChannel.AddString(buf);
				}
			m_cmbChannel.SetCurSel( min(m_pConfig->m_iChannel,m_pConfig->m_iChannels-1) );
			
			//-- data - whole file, selection list, current selection 
			if( m_pConfig->m_iDataMode==2 && m_bDataCurrent ) 
				m_selDataCurrent.SetCheck(true);
			else if( m_pConfig->m_iDataMode==1 && m_bDataList ) 
				m_selDataList.SetCheck(true);
			else 
				m_selDataFile.SetCheck(true);
				
			//-- Fill in open model names  
			m_edtBinCount.SetTextInt( m_pConfig->m_iBins );
			}

		//-----------------------------------------------------------------------------
		bool GetControlValues(  ) { 
			//-- Get Selected idealization channel
			m_pConfig->m_iChannel = max(m_cmbChannel.GetCurSel(),0);

			//-- Get selected data 
			if( m_selDataCurrent.GetCheck() ) 
				m_pConfig->m_iDataMode=2;
			else if( m_selDataList.GetCheck() )
				m_pConfig->m_iDataMode=1;
			else if( m_selDataFile.GetCheck() ) 
				m_pConfig->m_iDataMode=0;
			else { 
				MessageBox( m_hDlg, "You must select a data option", g_szTitleAmpHist, MB_OK );
				return false;
				}
				
			//-- bin count
			m_pConfig->m_iBins = m_edtBinCount.GetTextInt();			
			
			return true;
			}

		//-----------------------------------------------------------------------------
		// Events and messages 

		//-- [CANCEL]  
		void OnBtnCancel( ) { 
			EndDialog( m_hDlg, IDCANCEL );
			}
			
		//-- [SAVE] 
		void OnSave( ) { 
			if( GetControlValues() ) 
				EndDialog( m_hDlg, IDOK );
			}
			
		void DisableDataCurrent() { 
			m_bDataCurrent=false;
			}

		void DisableDataList() { 
			m_bDataList = false;
			}
	};

//-----------------------------------------------------------------------------
CVwAmpHistogram * g_pVwAmpHistogram;		// Modal - one and only instance

//-----------------------------------------------------------------------------
void OpenVwAmpHistogram( HWND hDlgVwData, CData * pData, bool bList, const char * pchList, 
							bool bSelect, int iSeg0, int iOffset0, int iSeg1, int iOffset1 ) {  
	
	//-- Attempt to lock data 
	if( ! pData->SetStatus(1) ) {
		MessageBox( hDlgVwData, "The data is currently in use by another task.", g_szTitleAmpHist, MB_OK );
		return;
		}
		
	//-- create dialogue to get task options.  Remove unavailable options.
	g_pVwAmpHistogram = new CVwAmpHistogram;
	if( ! bSelect || (iSeg0==iSeg1 && iOffset0>=iOffset1) || (iSeg0>iSeg1) ) 
		g_pVwAmpHistogram->DisableDataCurrent();	
	if( ! bList || pchList==NULL || strlen(pchList)==0 ) 
		g_pVwAmpHistogram->DisableDataList();		

	//-- create structure to pass option to / from dialogue.  Constructor sets defaults
	COptionsAmpHistogram * pOptions = new COptionsAmpHistogram;		
	g_pVwAmpHistogram->m_pConfig = pOptions;
	g_pVwAmpHistogram->m_pConfig->RegistryLoad();
	g_pVwAmpHistogram->m_pConfig->m_iChannels=pData->m_oConfig.ChannelCount();
	g_pVwAmpHistogram->m_pConfig->m_iChannel=min(pData->m_oConfig.ChannelCount(),g_pVwAmpHistogram->m_pConfig->m_iChannel);
	
	//-- present dialogue, execute if [OK]
	if( DialogBox( GetInstance(), (LPCTSTR)IDD_TASK_AMPHISTOGRAM, hDlgVwData, (DLGPROC)VwAmpHistogramDlgProc ) == IDOK ) { 
		pOptions->m_iBins = max( pOptions->m_iBins, 1 );
		pOptions->m_iDataMode = max( 0, pOptions->m_iDataMode );
	
		pOptions->RegistrySave();
		
		//-- Open the task & progress view 	
		CVwTaskAmpHistogram * pVwTask = new CVwTaskAmpHistogram( );
		pVwTask->m_pOptionsAmpHistogram = pOptions;  // transfer ownership of this heap object to task
		pVwTask->m_hDlgVwData = hDlgVwData;			// reference to data window 

		//-- dataset setup
		bool lSetSetup=false;
		if( pOptions->m_iDataMode==0 ) 
			lSetSetup=pVwTask->m_oSelSet.Setup( pData, pOptions->m_iChannel );
		else if( pOptions->m_iDataMode==1 )
			lSetSetup=pVwTask->m_oSelSet.Setup( pData, pOptions->m_iChannel, pchList );
		else if( pOptions->m_iDataMode==2 ) 
			lSetSetup=pVwTask->m_oSelSet.Setup( pData, pOptions->m_iChannel, iSeg0, iOffset0, iSeg1, iOffset1 );
			
		if( lSetSetup ) {						// if dataset was obtainable
			//-- Create separate thread to do idealization
			HWND hVwTask = pVwTask->CreateTaskWindow( GetClientWin(), GetChildClassName() );
			pVwTask->m_hThread = CreateThread( NULL, 0, TaskAmpHistogram, pVwTask, 0, NULL );
			}
		else 
			MessageBox( hDlgVwData, "Error setting data source", g_szTitleAmpHist, MB_OK);
		} 
	else 
		delete pOptions;
	pData->SetStatus(0);
	}

//-----------------------------------------------------------------------------
// Message Pump
BOOL CALLBACK VwAmpHistogramDlgProc( HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam) { 
	int wmId, wmEvent;
	
	switch( iMsg ) { 
		//----- DIALOG INIT -  Control Initialization
		case WM_INITDIALOG :		
			g_pVwAmpHistogram->m_hDlg=hDlg;			
			
			//-- Clear Existing  Values
			g_pVwAmpHistogram->InitializeControls();
			g_pVwAmpHistogram->SetControlValues();

			return TRUE;

		//----- MENU and CONTROL Commands
		case WM_COMMAND : 
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 

			switch( wmId ) { 
				case IDOK :					//-- [OK]
					g_pVwAmpHistogram->OnSave( );
					break;

				case IDCANCEL :				//-- [CANCEL]
					g_pVwAmpHistogram->OnBtnCancel( );
					break;
				}
				
			return TRUE;

		case WM_DESTROY : 
			delete g_pVwAmpHistogram;
			WindowListRemove( hDlg );
			return TRUE;
			
		default : 
			break;
		}
	return FALSE;
	} 

